/* MMUtilities.h */

#ifndef H_MMUtilities_H
#define H_MMUtilities_H

/*!
@header MMUtilities.h

@abstract Helper functions for ModuleManager.h calls.

@related ModuleManager.h MMConstants.h

*/

#include <CoreFoundation/CoreFoundation.h>
#include <ModuleManager/ModuleManager.h>

#ifdef __cplusplus
	extern "C" {
#endif


/*! 
@functiongroup MMUtilities
@discussion Assorted helper functions.
*/

/*!
@function MMU_URLCopyFileToFolder
@discussion Copies the file to the destination folder optionally giving it a new name.
If doReplace is kCFBooleanFalse, and the destination with the newly constructed file name exists, the file will not be written over. 
*/

extern MMErrorRef MMU_URLCopyFileToFolder( CFAllocatorRef theAllocator, CFURLRef theFileURL, CFURLRef theDestinationFolder, CFStringRef optNewName, CFBooleanRef doReplace );

/*!
@function MMU_URLCreateFolderInParent
@discussion Creates a subfolder in the parent's folder. If the folder already exists, it still returns a valid URL.
*/

extern CFURLRef MMU_URLCreateFolderInParent( CFAllocatorRef theAllocator, MMUUIDRef theParentID, CFStringRef theNewFolderName, MMErrorRef *outError );


/*!
@function MMU_URLTargetExists
@discussion Tests whether or not the resource (file, folder, response from an HTTP server, etc.) specified by theURL exists. If it does, the input URL is returned. If not, NULL.
*/

extern CFURLRef MMU_URLTargetExists( CFAllocatorRef theAllocator, CFURLRef theURL );

/*!
@function MMU_URLCopyPathData
@discussion Supplies non-NULL inputs outPOSIXPath, outHFSPath, and outBookmarkData with values.
*/

extern void MMU_URLCopyPathData( CFAllocatorRef theAllocator, CFURLRef theURL, CFStringRef *outPOSIXPath, CFStringRef *outHFSPath, CFDataRef *outBookmarkData );


/*!
@function MMU_URLCopyFileInfo
@discussion Returns file system information about the URL if it is available. 
*/

extern CFTypeRef MMU_URLCopyFileInfo( CFAllocatorRef theAllocator, CFURLRef theURL, CFStringRef theInfoKey, MMErrorRef *outError );


/*!
@function MMU_NumberCreateWithCFIndex
@discussion Convenience wrapper for CFNumberCreation.
*/

extern CFNumberRef MMU_NumberCreateWithCFIndex( CFAllocatorRef theAllocator, const CFIndex theNumber );


/*!
@function MMU_NumberGetCFIndexValue
@discussion Convenience wrapper for extracting an integer from the number.
*/


CFIndex MMU_NumberGetCFIndexValue( CFNumberRef theNumber );


/*!
@defined MMU_SafeRelease
@discussion Wrapper for CFRelease() which detects a NULL input. An optional message is printed if input theCFType is NULL.
*/

/* Should be:
extern void MMU_SafeRelease( CFTypeRef theCFType, CFStringRef theMessageIfNull );
*/

#define MMU_SafeRelease( theCFType, theMessage ) {	\
	if( theCFType != NULL ) {	\
		CFRelease( theCFType );	\
		theCFType = NULL;	\
	}	\
	else {	\
		if( theMessage != NULL ) {	\
			CFShow( theMessage );	\
		}	\
	}	\
}


/*!
@function MMU_ShowType
@discussion If the input is a single CFTypeRef, prints it. If the first input is a format string, prints the values accordingly.
*/

extern void MMU_ShowType( CFTypeRef theCFTypeOrFormatString, ... );


/*!
@function MMU_ErrorGetCode
@discussion Gets the error number (translated to an integer) contained in <b>theError</b>.
*/

extern CFIndex MMU_ErrorGetCode( MMErrorRef theError );


#ifdef __cplusplus
}
#endif

#endif /*H_MMUtilities_H*/
