/* ModuleManager.h */

#ifndef H_ModuleManager_H
#define H_ModuleManager_H

/*!
@header ModuleManager.framework

@abstract Declarations for functions available in ModuleManager.framework.

@version 0.5.3

@related MMConstants.h, MMUtilities.h

@discussion <b>ModuleManager.framework</b> provides access to an XML <a href="x-man-page://5/plist">property list</a> file which describes the contents of script libraries. 
The idea of using a property list storage format for script libraries is that its form be codified at a basic level so users and developers can take advantage of a known format.
Another requirement is that it be extensible so clients can add extra facilities while maintaining compatibility with the basic interchange format.
ModuleManager subscribes to these concepts.

ModuleManager.framework uses the idea that script libraries are located in subfolders of folders named "Modules" located in the Scripts folders of the user, local, and network domains. 
It also entertains the notion that library descriptions don't have to be folder-based.
They can describe a hierarchy of resources which may be anywhere on a disk or the internet. 

ModuleManager's programming interface is divided into the following sections:

<ul>
<li><b>ModuleManager</b>: Document activation control and r/o variables available prior to document activation.</li>
<li><b>MMDocument</b>: Top level document functions.</li>
<li><b>MMLibrary</b>: For manipulating libraries.</li>
<li><b>MMLibraryDescription</b>: Used for creating libraries.</li>
<li><b>MMFile</b>: Access to file information.</li>
<li><b>MMProperty</b>: Access to library properties information.</li>
<li><b>MMSearchConfiguration</b>: For manipulating search configurations.</li>
<li><b>MMSearchConfigurationDescription</b>: Used for creating search configurations.</li>
<li><b>MMError</b>: Error reporting.</li>
</ul>

ModuleManager is a layered technology.

The bottom layer (outlined below) does not make calls above the CoreFoundation framework APIs and only uses CoreFoundation types.
This makes it easy to create a wrapper interface with object-oriented languages.
An auxilliary file, MMUtilities.h, provides helper functions which use C-language primatives.

The next layer interfaces with CoreServices level technologies such as Aliases and AppleEvents.
Its function declarations can be found in the MMServices.h file of the framework Headers folder.
In order to use them, you must <code>#include</code> or <code>#import</code> the header as: <code>&lt;ModuleManager/MMServices.h&gt;</code>.

@CFBundleIdentifier info.framework.ModuleManager
@availability Mac OS X 10.5 and later.
@author <a href="mailto:philip.aker\@shaw.ca">Philip Aker</a>
@copyright 2010, Philip Aker. All Rights Reserved.


*/

#include <CoreFoundation/CoreFoundation.h>
#include <ModuleManager/MMConstants.h>

#ifdef __cplusplus
	extern "C" {
#endif


/*! 
@functiongroup ModuleManager
@discussion Top level controller.
*/

/*!
@function ModuleManagerLoadDocument
@param theAllocator A standard CoreFoundation allocator.
@param theOptions A CFDictionaryRef for initialization options. May be NULL. The only option currently implemented is keyed with CFSTR( "InstallExamples" ) and a CFBoolean value.
@result An MMErrorRef on error or NULL on success.
@discussion Loads the document (the property list file) making it available for use by all the other functions in the framework.
*/

extern MMErrorRef ModuleManagerLoadDocument( CFAllocatorRef theAllocator, CFDictionaryRef theOptions );


/*!
@function ModuleManagerSaveDocument
@discussion Saves the current state of the document to disk.
*/

extern MMErrorRef ModuleManagerSaveDocument( CFAllocatorRef theAllocator );


/*!
@function ModuleManagerGetModulesFolderName
@discussion Returns the folder name for the module in each domain (currently "Modules").
For beta purposes, the name is "Modules.&lt;Version&gt;".
This can be changed by altering the value for the key "ModuleDefaultFolderName" in the framework's Info.plist.
The folder name must have a length of at least 1 character.
*/

extern CFStringRef ModuleManagerGetModulesFolderName( void );


/*!
@function ModuleManagerGetDocumentPropertyListID
@discussion Returns the property list identifier used to access the shared property list.
For beta purposes, the identifier is "info.library.module.&lt;Version&gt;." -- meaning that the file is located in the /Users/Shared directory.
This can be changed by altering the value for the key "ModuleDocumentPropertyListID" in the framework's Info.plist.
The identifier must have a length of at least 16 characters.
If for any reason, retrieval of the default identifier fails, it will be hard-wired to "info.library.module".
*/

extern CFStringRef ModuleManagerGetDocumentPropertyListID( void );

/*!
@function ModuleManagerGetDocumentFileExtension
@discussion Returns the extension of the module document (default is 'mlix' ).
*/

extern CFStringRef ModuleManagerGetDocumentFileExtension( void );


/*!
@function ModuleManagerGetModulesFolderURLForUserDomain
@discussion The file URL for ~/Library/Scripts/${Modules}/ (i.e. whatever ModuleManagerGetModulesFolderName() returns). Module initialization will fail if this URL can't be created.
*/

extern CFURLRef ModuleManagerGetModulesFolderURLForUserDomain( void );

/*!
@function ModuleManagerGetModulesFolderURLForLocalDomain
@discussion The file URL for /Library/Scripts/${Modules}/ (i.e. whatever ModuleManagerGetModulesFolderName() returns).
*/

extern CFURLRef ModuleManagerGetModulesFolderURLForLocalDomain( void );

/*!
@function ModuleManagerGetModulesFolderURLForNetworkDomain
@discussion Could be the file URL for /Network/Library/Scripts/${Modules}/ (i.e. whatever ModuleManagerGetModulesFolderName() returns), but NULL when the user is not logged in to a network.
*/

extern CFURLRef ModuleManagerGetModulesFolderURLForNetworkDomain( void );


/*! 
@functiongroup MMDocument
@discussion Top level controller.
*/

/*!
@function MMDocumentGetVersion
@discussion Returns the current version of the document storage format.
*/

extern CFStringRef MMDocumentGetVersion( void );

/*!
@function MMDocumentGetDisabledFolderSubstrings
@discussion Returns an array of substrings whose presence in the name of a folder determines whether or not it is enabled for searching.
*/

extern CFMutableArrayRef MMDocumentGetDisabledFolderSubstrings( void );


/*!
@function MMDocumentSetDisabledFolderSubstrings
@discussion Sets the disabled folder substrings array.
*/

extern void MMDocumentSetDisabledFolderSubstrings( CFArrayRef theArray );


/*!
@function MMDocumentRegisterLibrary
@param theAllocator A standard CoreFoundation allocator.
@param theLibraryDescriptionID A reference to a LibraryDescription created with <a href="#MMLibraryDescriptionCreateMinimal">MMLibraryDescriptionCreateMinimal</a>().
@param optOutLibraryID An optional output containing the registered library id. May be NULL. This is the same <em>thePermanentID</em> parameter used with MMLibraryDescriptionCreateMinimal().
@result An MMErrorRef on error or NULL on success.
@discussion Add a library to the collection of libraries for the module. The basic items in a library are:

<dl>
<dt>BaseURIString</dt>
<dd>Fixed text. The base path of the default libraries in their respective domains, but otherwise need not be present. Tilde expansion may be required for the user domain.</dd>
<dt>Children</dt>
<dd>An array of library IDs which were registered with this library's ID as their parent ID. May be empty.</dd>
<dt>Files</dt>
<dd>Array of IDs. These specify keys in the top-level Map container for the string representation values of URLs registered in the library. May be empty.</dd>
<dt>Flags</dt>
<dd>CFNumberRef. Stored as OR'd enumeration values. The utility functions <b>MMU_NumberCreateWithCFIndex</b> and <b>MMU_NumberGetCFIndexValue</b> are available for dealing with flag values.
	<dl>
	<dt>Values:</dt>
	<dd><b>kMMLibraryFlagsNone</b></dd>
	<dd><b>kMMLibraryFlagPermitOverrides</b></dd>
	<dd><b>kMMLibraryFlagSearchRecursively</b></dd>
	<dd><b>kMMLibraryFlagTrackAliases</b></dd>
	<dd><b>kMMLibraryFlagIsReserved</b></dd>
	</dl>
</dd>
<dt>Kind</dt>
<dd>The Library type identifier.</dd>
<dt>ParentID</dt>
<dd>A unique identifier. An <b>MMUUIDRef</b> supplied by theParentLibraryID input parameter in MMLibraryDescriptionCreateMinimal().</dd>
<dt>PropertyID</dt>
<dd>A unique identifier. An <b>MMUUIDRef</b> auto-supplied by ModuleManager.
It is the key to a property container in the top-level Properties container.
</dd>
<dt>URLMapID</dt>
<dd>A unique identifier. Specifies a key in the top-level Map container for the string representation value of the URL directly associated with this library (most often a folder on disk).
Will not be present if there is no associated URL.</dd>
</dl>

<p class="xmpl">Example of a library registered with a minimal description.</p>

The supplied library description for this example was created with an MMLibraryDescriptionCreateMinimal() call. No additional values were added.
The library title is stored in the top-level Properties container and accessed with the auto-generated PropertiesID value. 

<pre>
&lt;key&gt;com.example.empty.library&lt;/key&gt;
&lt;dict&gt;
    &lt;key&gt;<b>Children</b>&lt;/key&gt;
    &lt;array/&gt;
    &lt;key&gt;<b>Files</b>&lt;/key&gt;
    &lt;array/&gt;
    &lt;key&gt;<b>Flags</b>&lt;/key&gt;
    &lt;integer&gt;0&lt;/integer&gt;
    &lt;key&gt;<b>Kind</b>&lt;/key&gt;
    &lt;string&gt;Library&lt;/string&gt;
    &lt;key&gt;<b>ParentID</b>&lt;/key&gt;
    &lt;string&gt;info.module.library.id.local&lt;/string&gt;
    &lt;key&gt;<b>PropertyID</b>&lt;/key&gt;
    &lt;string&gt;21190e8bd367fea65df6dd2b3a0acd50&lt;/string&gt;
</pre>
*/

extern MMErrorRef MMDocumentRegisterLibrary( CFAllocatorRef theAllocator, MMUUIDRef theLibraryDescriptionID, MMUUIDRef *optOutLibraryID );


/*!
@function MMDocumentRemoveLibrary
@discussion Deletes the specified library.
*/

extern MMErrorRef MMDocumentRemoveLibrary( CFAllocatorRef theAllocator, MMUUIDRef theLibraryID );


/*!
@function MMDocumentCopyLibraryIDs
@discussion Copies all library ids currently installed in the module.
*/

extern CFMutableArrayRef MMDocumentCopyLibraryIDs( CFAllocatorRef theAllocator, MMErrorRef *outError );


/*!
@function MMDocumentCopyLibraryIDsWithTitle
@discussion Copies all library ids with the specified title.
*/

extern CFMutableArrayRef MMDocumentCopyLibraryIDsWithTitle( CFAllocatorRef theAllocator, CFStringRef theLibraryTitle, MMErrorRef *outError );

/*!
@function MMDocumentCopyLibraryIDsWithTitleContainingSubstring
@discussion Copies all library ids with the specified substring in the library title.
*/

extern CFMutableArrayRef MMDocumentCopyLibraryIDsWithTitleContainingSubstring( CFAllocatorRef theAllocator, CFStringRef theSubstring, MMErrorRef *outError );


/*!
@function MMDocumentGetSearchConfigurationLastSelected
@discussion Get the active search configuration ID.
*/

MMUUIDRef MMDocumentGetSearchConfigurationLastSelected( void );


/*!
@function MMDocumentSetSearchConfigurationLastSelected
@discussion Set the active search configuration. Should be called in response to a user action.
*/

extern MMErrorRef MMDocumentSetSearchConfigurationLastSelected( CFAllocatorRef theAllocator, MMUUIDRef theSearchConfigurationID );


/*!
@function MMDocumentRegisterSearchConfiguration
@discussion Adds the referenced SearchConfiguration to the module replacing any existing configuration with the same id.
The input parameter theSearchConfigurationDescriptionID is released during this process and no longer valid after the call completes.
*/

extern MMErrorRef MMDocumentRegisterSearchConfiguration( CFAllocatorRef theAllocator, MMUUIDRef theSearchConfigurationDescriptionID );

/*!
@function MMDocumentRemoveSearchConfiguration
@discussion Deletes the specified SearchConfiguration. 
*/

extern MMErrorRef MMDocumentRemoveSearchConfiguration( CFAllocatorRef theAllocator, MMUUIDRef theSearchConfigurationID );


/*!
@function MMDocumentCopySearchConfigurationIDs
@discussion Copies all search configuration ids currently in the document.
*/

CFMutableArrayRef MMDocumentCopySearchConfigurationIDs( CFAllocatorRef theAllocator, MMErrorRef *outError );


/*!
@function MMDocumentCopyFileIDs
@discussion Copy the ids of all files in the document.
*/

extern CFMutableArrayRef MMDocumentCopyFileIDs( CFAllocatorRef theAllocator, MMErrorRef *outError );


/*!
@function MMLibraryRegisterSubfolders
@discussion Registers the immediate subfolders (and their files) of the specified library.
*/

extern MMErrorRef MMLibraryRegisterSubfolders( CFAllocatorRef theAllocator, MMUUIDRef theLibraryID );


/*! 
@functiongroup MMProperty
@discussion Access to property containers.
*/


/*!
@function MMPropertyGetValue
@discussion Gets the value of the item specified by 'theKey' in the property container.
*/

extern MMTypeRef MMPropertyGetValue( CFAllocatorRef theAllocator, MMUUIDRef thePropertyID, CFStringRef theKey, MMErrorRef *outError );


/*!
@function MMPropertySetValue
@discussion Sets the value of the item specified by 'theKey' in the property container.
*/

extern MMErrorRef MMPropertySetValue( CFAllocatorRef theAllocator, MMUUIDRef thePropertyID, CFStringRef theKey, MMTypeRef theValue );


/*! 
@functiongroup MMLibrary
@discussion Interfacing with libraries.
*/

/*!
@function MMLibraryGetValue
@discussion Gets a value from the top level library container. Do not release the value obtained.
*/

extern MMTypeRef MMLibraryGetValue( CFAllocatorRef theAllocator, MMUUIDRef theLibraryID, CFStringRef theItemKey, MMErrorRef *outError );


/*!
@function MMLibrarySetValue
@discussion Sets the value of the specified library item.
*/

extern MMErrorRef MMLibrarySetValue( CFAllocatorRef theAllocator, MMUUIDRef theLibraryID, CFStringRef theItemKey, MMTypeRef theValue );


/*!
@function MMLibraryGetPropertyID
@discussion Gets the id of the Property container associated with the specified library.
*/

extern MMUUIDRef MMLibraryGetPropertyID( CFAllocatorRef theAllocator, MMUUIDRef theLibraryID, MMErrorRef *outError );


/*!
@function MMLibraryGetURLMapID
@discussion Gets the id of the folder associated with the specified library (if it exists).
*/

extern MMUUIDRef MMLibraryGetURLMapID( CFAllocatorRef theAllocator, MMUUIDRef theLibraryID, MMErrorRef *outError );


/*!
@function MMLibraryPropertyGetValue
@discussion Gets the value of the specifed item in the Property container associated with the library.
*/

extern MMTypeRef MMLibraryPropertyGetValue( CFAllocatorRef theAllocator, MMUUIDRef theLibraryID, CFStringRef thePropertyKey, MMErrorRef *outError );


/*!
@function MMLibraryPropertySetValue
@discussion Sets an item value in the specified library Properties container.
*/

extern MMErrorRef MMLibraryPropertySetValue( CFAllocatorRef theAllocator, MMUUIDRef theLibraryID, CFStringRef thePropertyKey, MMTypeRef theValue );



/*!
@function MMLibraryGetFiles
@discussion Gets the array of file ids contained in the specified library. Returns NULL (and no error) if there are none.
*/

extern CFMutableArrayRef MMLibraryGetFiles( CFAllocatorRef theAllocator, MMUUIDRef theLibraryID, MMErrorRef *outError );


/*!
@function MMLibraryGetFlags
@discussion Gets the value of the flags for the specified library. Do not release the returned number.
*/

static inline CFNumberRef MMLibraryGetFlags( CFAllocatorRef theAllocator, MMUUIDRef theLibraryID, MMErrorRef *outError ) {
	return (CFNumberRef)MMLibraryGetValue( theAllocator, theLibraryID, kMMKeyFlags, outError );
}



/*!
@function MMLibrarySetFlags
@discussion Sets the flags value in the specified library.
*/

static inline MMErrorRef MMLibrarySetFlags( CFAllocatorRef theAllocator, MMUUIDRef theLibraryID, CFNumberRef theNumber ) {
	return MMLibrarySetValue( theAllocator, theLibraryID, kMMKeyFlags, theNumber );
}


/*!
@function MMLibraryRegisterFileID
@discussion If a URL has already been assigned an id in another library, you can use that id to register the file in this library as well.
*/

extern MMErrorRef MMLibraryRegisterFileID( CFAllocatorRef theAllocator, MMUUIDRef theLibraryID, MMUUIDRef theUUID );


/*!
@function MMLibraryRegisterURL
@discussion Adds file information about the input file to the document's Files container and the input library's Files list.
If file information for the input file already exists, the input library id is added to its Owners list.
*/

extern MMUUIDRef MMLibraryRegisterURL( CFAllocatorRef theAllocator, MMUUIDRef theLibraryID, CFURLRef theURL, MMErrorRef *outError );


/*!
@function MMLibraryUnregisterFileID
@discussion Removes the file from the library's Files list. Removes the library as an owner of the file in the top level Files container.
The file is not removed from the top level Files container. If file represents an existing file on disk, is not touched.
*/

extern MMErrorRef MMLibraryUnregisterFileID( CFAllocatorRef theAllocator, MMUUIDRef theLibraryID, MMUUIDRef theFileID );


/*!
@function MMLibraryUnregisterURL
@discussion Removes the URL's associated ID from the library's Files list. Removes the library as an owner of the file in the top level Files container.
The file is not removed from the top level Files container. If file represents an existing file on disk, is not touched.
*/

extern MMErrorRef MMLibraryUnregisterURL( CFAllocatorRef theAllocator, MMUUIDRef theLibraryID, CFURLRef theURL );



/*! 
@functiongroup MMLibraryDescription
@discussion Interfacing with library descriptions. Used in library creation.
*/

/*!
@function MMLibraryDescriptionCreateMinimal
@discussion Creates the minimal library description suitable as input for registering a library.
Once created, you may use the assorted MMLibraryDescription functions to add additional information to be associated with the library.
The MMLibraryDescriptionSetFolderInfoWithURL() function is used to associate a library with a folder.

*/

extern MMUUIDRef MMLibraryDescriptionCreateMinimal( CFAllocatorRef theAllocator, MMUUIDRef theParentLibraryID, MMUUIDRef thePermanentID, CFStringRef theNewLibraryTitle, MMErrorRef *outError );

/*!
@function MMLibraryDescriptionGetValue
@discussion Gets an item value in the specified library description.
*/

extern MMTypeRef MMLibraryDescriptionGetValue( CFAllocatorRef theAllocator, MMUUIDRef theLibraryDescriptionID, CFStringRef theItemKey, MMErrorRef *outError );

/*!
@function MMLibraryDescriptionContainerGetValue
@discussion Gets an item value in the specified library description container.
*/

extern MMTypeRef MMLibraryDescriptionContainerGetValue( CFAllocatorRef theAllocator, MMUUIDRef theLibraryDescriptionID, CFStringRef theContainerKey, CFStringRef theItemKey, MMErrorRef *outError );


/*!
@function MMLibraryDescriptionPropertyGetValue
@discussion Gets an item value from the specified library description Properties container.
*/

static inline MMTypeRef MMLibraryDescriptionPropertyGetValue( CFAllocatorRef theAllocator, MMUUIDRef theLibraryDescriptionID, CFStringRef thePropertyKey, MMErrorRef *outError ) {
	return MMLibraryDescriptionContainerGetValue( theAllocator, theLibraryDescriptionID, kMMLibraryDescriptionKeyProperties, thePropertyKey, outError );
}

/*!
@function MMLibraryDescriptionFileGetValue
@discussion Gets an item value from the specified library description Files container.
*/

static inline MMTypeRef MMLibraryDescriptionFileGetValue( CFAllocatorRef theAllocator, MMUUIDRef theLibraryDescriptionID, CFStringRef theFileInfoKey, MMErrorRef *outError ) {
	return MMLibraryDescriptionContainerGetValue( theAllocator, theLibraryDescriptionID, kMMKeyFiles, theFileInfoKey, outError );
}

/*!
@function MMLibraryDescriptionFolderInfoGetValue
@discussion Gets an item value from the specified library description FolderInfo container.
*/

static inline MMTypeRef MMLibraryDescriptionFolderInfoGetValue( CFAllocatorRef theAllocator, MMUUIDRef theLibraryDescriptionID, CFStringRef theFileInfoKey, MMErrorRef *outError ) {
	return MMLibraryDescriptionContainerGetValue( theAllocator, theLibraryDescriptionID, kMMKeyFolderInfo, theFileInfoKey, outError );
}


/*!
@function MMLibraryDescriptionGetFlags
@discussion Gets the flag value from the specified library description. Do not release the returned value.
*/

static inline CFNumberRef MMLibraryDescriptionGetFlags( CFAllocatorRef theAllocator, MMUUIDRef theLibraryDescriptionID, MMErrorRef *outError ) {
	return (CFNumberRef)MMLibraryDescriptionGetValue( theAllocator, theLibraryDescriptionID, kMMKeyFlags, outError );
}

/*!
@function MMLibraryDescriptionSetValue
@discussion Sets the value of an item in specified library description.
*/

extern MMErrorRef MMLibraryDescriptionSetValue( CFAllocatorRef theAllocator, MMUUIDRef theLibraryDescriptionID, CFStringRef theItemKey, MMTypeRef theValue );

/*!
@function MMLibraryDescriptionContainerSetValue
@discussion Sets the value of an item in specified library description container.
*/

extern MMErrorRef MMLibraryDescriptionContainerSetValue( CFAllocatorRef theAllocator, MMUUIDRef theLibraryDescriptionID, CFStringRef theContainerKey, CFStringRef theItemKey, MMTypeRef theValue );

/*!
@function MMLibraryDescriptionPropertySetValue
@discussion Sets the value of an item in specified library description Properties container.
*/

static inline MMErrorRef MMLibraryDescriptionPropertySetValue( CFAllocatorRef theAllocator, MMUUIDRef theLibraryDescriptionID, CFStringRef thePropertyKey, MMTypeRef theValue ) {
	return MMLibraryDescriptionContainerSetValue( theAllocator, theLibraryDescriptionID, kMMLibraryDescriptionKeyProperties, thePropertyKey, theValue );
}

/*!
@function MMLibraryDescriptionFileSetValue
@discussion Sets the value of an item in specified library description Files container.
*/

static inline MMErrorRef MMLibraryDescriptionFileSetValue( CFAllocatorRef theAllocator, MMUUIDRef theLibraryDescriptionID, CFStringRef theFileInfoKey, MMTypeRef theValue ) {
	return MMLibraryDescriptionContainerSetValue( theAllocator, theLibraryDescriptionID, kMMKeyFiles, theFileInfoKey, theValue );
}


/*!
@function MMLibraryDescriptionSetFlags
@discussion Sets the value of the flags in the specified library description.
*/

static inline MMErrorRef MMLibraryDescriptionSetFlags( CFAllocatorRef theAllocator, MMUUIDRef theLibraryDescriptionID, CFNumberRef theFlags ) {
	return MMLibraryDescriptionSetValue( theAllocator, theLibraryDescriptionID, kMMKeyFlags, theFlags );
}


/*!
@function MMLibraryDescriptionFolderInfoSetValue
@discussion Sets the value of an item in specified library description FolderInfo container. 
*/

static inline MMErrorRef MMLibraryDescriptionFolderInfoSetValue( CFAllocatorRef theAllocator, MMUUIDRef theLibraryDescriptionID, CFStringRef theFileInfoKey, MMTypeRef theValue ) {
	return MMLibraryDescriptionContainerSetValue( theAllocator, theLibraryDescriptionID, kMMKeyFolderInfo, theFileInfoKey, theValue );
}


/*!
@function MMLibraryDescriptionSetFolderInfoWithURL
@discussion Sets the value of the URLString in the FolderInfo container.
Optionally, depending on the value of theFlags parameter, calculates and set the values of the POSIXPath, HFSPath, and BookmarkData keys.
*/

extern MMErrorRef MMLibraryDescriptionSetFolderInfoWithURL( CFAllocatorRef theAllocator, MMUUIDRef theLibraryDescriptionID, CFURLRef theURL, CFNumberRef theFlags );


/*! 
@functiongroup MMFile
@discussion Access to file containers.
*/


/*!
@function MMFileGetValue
@discussion Gets the value of the item specified by 'theKey' in the file container.
*/

extern MMTypeRef MMFileGetValue( CFAllocatorRef theAllocator, MMUUIDRef theFileID, CFStringRef theKey, MMErrorRef *outError );


/*!
@function MMFileGetURL
@discussion Gets the URL string associated with the specified file id.
*/

extern CFStringRef MMFileGetURL( MMUUIDRef theFileID );


/*!
@function MMFileSetValue
@discussion Sets the value of the item specified by 'theKey' in the file container.
*/

extern MMErrorRef MMFileSetValue( CFAllocatorRef theAllocator, MMUUIDRef theFileID, CFStringRef theKey, MMTypeRef theValue );


/*! 
@functiongroup MMSearchConfiguration
@discussion Interfacing with search configurations.
*/


/*!
@function MMSearchConfigurationGetValue
@discussion Copies the value of an item in the specified search configuration.
*/

extern MMTypeRef MMSearchConfigurationGetValue( CFAllocatorRef theAllocator, MMUUIDRef theSearchConfigurationID, CFStringRef theItemKey, MMErrorRef *outError );


/*!
@function MMSearchConfigurationSetValue
@discussion Sets the value of an item in specified search configuration.
*/

extern MMErrorRef MMSearchConfigurationSetValue( CFAllocatorRef theAllocator, MMUUIDRef theSearchConfigurationID, CFStringRef theItemKey, MMTypeRef theValue );


/*!
@function MMSearchConfigurationGetLibraryIDs
@discussion Copies the array of library ids in the specified search configuration.
*/

static inline CFMutableArrayRef MMSearchConfigurationGetLibraryIDs( CFAllocatorRef theAllocator, MMUUIDRef theSearchConfigurationID, MMErrorRef *outError ) {
	return (CFMutableArrayRef)MMSearchConfigurationGetValue( theAllocator, theSearchConfigurationID, kMMKeyLibraryIDs, outError );
}


/*! 
@functiongroup MMSearchConfigurationDescription
@discussion Search configuration creation functions.
*/


/*!
@function MMSearchConfigurationDescriptionCreate
@discussion Creates a minimal search configuration description. The parameters theConfigurationTitle and thePermanentID are required.
*/

extern MMErrorRef MMSearchConfigurationDescriptionCreate( CFAllocatorRef theAllocator, CFStringRef theConfigurationTitle, MMUUIDRef thePermanentID, CFArrayRef theLibraryIDs );

/*!
@function MMSearchConfigurationDescriptionGetValue
@discussion Gets the value in the search configuration description for the specified key.
*/

extern MMTypeRef MMSearchConfigurationDescriptionGetValue( CFAllocatorRef theAllocator, MMUUIDRef theSearchConfigurationID, CFStringRef theKey, MMErrorRef *outError );

/*!
@function MMSearchConfigurationDescriptionSetValue
@discussion Sets the value in the search configuration description for the specified key.
*/

extern MMErrorRef MMSearchConfigurationDescriptionSetValue( CFAllocatorRef theAllocator, MMUUIDRef theSearchConfigurationID, CFStringRef theKey, MMTypeRef theValue );


/*! 
@functiongroup MMError
@discussion Error handling
*/


/*!
@function MMErrorCreate
@discussion Creates an error description with the specified inputs. The supplied domain is one of <b>kMMErrorDomainOSStatus</b> or <b>kMMErrorDomainPOSIX</b>.
The text (possibly resulting from an evaluation of theDescriptionFormat parameter and subsequent arguments) is available from <b>MMErrorGetDescription</b>().
Parameter <b>theDescriptionFormat</b> may be NULL.
*/

extern MMErrorRef MMErrorCreate( CFAllocatorRef theAllocator, CFStringRef theDomain, CFNumberRef theErrorNumber, CFStringRef theDescriptionFormat, ... );

/*!
@function MMErrorShow
@discussion Prints the values contained in <b>theError</b> to standard error and optionally displays an additional message.
*/

extern void MMErrorShow( MMErrorRef theError, CFStringRef theFormat, ... );

/*!
@function MMErrorGetDomain
@discussion Gets the domain string contained in <b>theError</b>.
*/

extern CFStringRef MMErrorGetDomain( MMErrorRef theError );

/*!
@function MMErrorGetNumber
@discussion Gets the error number contained in <b>theError</b>.
*/

extern CFNumberRef MMErrorGetNumber( MMErrorRef theError );

/*!
@function MMErrorGetDescription
@discussion Returns the text description contained in <b>theError</b> or NULL if none is available.
*/

extern CFStringRef MMErrorGetDescription( MMErrorRef theError );


/*!
@function MMErrorGetParamErr
@discussion Returns a CFNumberRef created with 'paramErr' value. Often used with <em>MMErrorCreate</em>().
*/

extern CFNumberRef MMErrorGetParamErr( void );


/*!
@function MMErrorGetCoreErr
@discussion Returns a CFNumberRef created with 'coreFoundationUnknownErr' value. Often used with <em>MMErrorCreate</em>().
*/

extern CFNumberRef MMErrorGetCoreErr( void );

#ifdef __cplusplus
}
#endif

#endif /*H_ModuleManager_H*/
